import json

import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.impute import SimpleImputer

from TOOLS.funciones import *


def do_exporatory_analysis():
    # Carga de datos
    # Obtenindo datos etiquetados y formateandolos
    with open('DATA/LABELED_DATASET/wc_labeldataset_bck_.json', 'r', encoding='utf-8') as f:
        # Cargar los datos desde el archivo JSON en un diccionario
        data = json.load(f)
    # Convertir el diccionario en un DataFrame de Pandas
    df_labeleddata = pd.DataFrame(data['wc_labeldataset_bck'])

    # 4 Analisis exploratorio
    # 4.1 Exploración del conjunto de datos
    print(df_labeleddata.info())
    # Explorando las primeras filas
    print(df_labeleddata.head())
    # Resumen estadístico
    print(df_labeleddata.describe())

    # 7.1.5 Exclusiones
    #   * No aportan valor al estudio (ej: id o id_spot)
    #   * Son datos que no se utilizan ya que se usó otra columna para el etiquetado (ej: velmedia)
    print("7.1.4 Exclusiones \n")
    print("Eliminando características que: \n"
          " * No aportan valor al estudio (ej: id o id_spot) \n"
          " * Son datos redundantes no seleccionados para el etiquetado (ej: velmedia)\n"
          " * Auxiliares del formulario web no relevantes (ej: labeled)\n")
    df_labeleddata = df_labeleddata.drop(columns=['id', 'id_spot', 'velmedia', 'labeled'])

    # 7.1.6 Transformaciones de datos
    # Actualizar los valores de la columna 'racha' donde sea 88 a None
    df_labeleddata['racha'] = df_labeleddata['racha'].replace(88, None)
    # Cambiar el tipo de datos de la columna dir a int64
    df_labeleddata['dir'] = df_labeleddata['dir'].astype('int64')
    # Convertir las direcciones de las rachas de grados a cardinales
    df_labeleddata['dir_card'] = df_labeleddata['dir'].apply(lambda x: grados_a_cardinal(x, True))
    # Convertir las direcciones de viento de grados a cardinales
    df_labeleddata['direccion_viento_card'] = df_labeleddata['direccion_viento'].apply(lambda x: grados_a_cardinal(x))
    # Convertir las direcciones del oleaje de grados a cardinales
    df_labeleddata['direccion_oleaje_card'] = df_labeleddata['direccion_oleaje'].apply(lambda x: grados_a_cardinal(x))
    # Convertir fecha a tipo datetime
    df_labeleddata['fecha'] = pd.to_datetime(df_labeleddata['fecha'])

    # 7.1.7 Codificaciones
    # Codificación nivel_rider de categórico a numérico para que entre la corr_matrix
    df_labeleddata['id_perfil_rider'] = df_labeleddata['perfil_rider']. \
        map({'Novel [<2 años]': 1, 'Avanzado [>2 años]': 2, 'Pro': 3})

    # 7.1.8 Discretizaciones
    # Creo nuevas columnas mes y año
    df_labeleddata['mes'] = df_labeleddata['fecha'].dt.strftime('%b')
    df_labeleddata['año'] = df_labeleddata['fecha'].dt.strftime('%Y')

    # 7.2 Preprocesamiento y gestión de características
    # 7.2.1 Valores nulos
    print()
    df_labeleddata = tratar_nulos(df_labeleddata)

    # 7.2.2 Varianza estadística ( Se descartan las varianzas inferiores a 0.01)
    print("7.2.2 Varianza estadistica\n")
    df_labeleddata = tratar_varianzas(df_labeleddata)
    del df_labeleddata['deporte']
    del df_labeleddata['modalidad']

    # 7.2.3 Revisión de clases objetivo
    print("7.2.3 Revisión de clases objetivo\n")
    # Visión general de las variables objetivo
    print(df_labeleddata[["sail_size", "wind_board_size", "valoracion"]].describe().round(0))
    # Calculando Proporicion de clases
    proporcion_clases(df_labeleddata,
                      ['sail_size', 'wind_board_size', 'valoracion'])

    # 7.2.4 Balanceo de clases objetivo
    # Oversampling sail_size
    df_labeleddata = oversample_variable(df_labeleddata, 'sail_size')
    # Undersampling valoracion
    df_labeleddata = oversample_variable(df_labeleddata, 'valoracion')
    # Frecuencias de las dimensiones en el conjunto de datos
    proporcion_clases(df_labeleddata,
                      ['sail_size', 'wind_board_size', 'valoracion'])

    # 7.2.5 Visión general de las variables objetivo (Comparativa)
    print(df_labeleddata[["sail_size", "wind_board_size", "valoracion"]].describe().round(0))

    # 7.2.6 Estudio de las frecuencias de las variables
    # Figura 9: Frecuencias de las dimensiones numéricas
    exploracion_univariada(df_labeleddata.drop(columns=['direccion_corriente',
                                                        'direccion_viento',
                                                        'direccion_oleaje',
                                                        'dir',
                                                        'id_perfil_rider',
                                                        'velocidad_viento']))
    # Figura 10: Frecuencias de las dimensiones no numéricas
    str_exploracion_univariada(df_labeleddata.drop(columns='fecha'))

    # 7.2.7 Correlación entre las variables
    # Defino una matriz de correlación para las variables numéricas
    corr_matrix = df_labeleddata.select_dtypes(include=['float64', 'int']) \
        .corr(method='pearson')
    # Grafico la matriz de correlación
    # Figura 11: Matriz de correlación Pearson de las variables continuas
    plot_corr_matrix(corr_matrix)

    # 7.2.8 Relación valoracíón vs variables
    # Figura 12: Realación valoración con algunas variables.
    plot_versus(df_labeleddata, 'valoracion', 'altura_oleaje')
    plot_versus(df_labeleddata, 'valoracion', 'velocidad_viento')
    plot_versus(df_labeleddata, 'direccion_viento_card', 'valoracion', None, 'y')
    plot_versus(df_labeleddata, 'direccion_oleaje_card', 'valoracion', None, 'y')
    plot_versus(df_labeleddata, 'valoracion', 'mes', None, 'x')
    plot_versus(df_labeleddata, 'valoracion', 'año', None, 'x')

    # Relacionando perfil y velocidad viento, altura ola y direccion viento con valoracion por perfiles
    sns.set(style="whitegrid")
    plt.figure(figsize=(10, 6))
    df_labeleddata['discretized_wind_speed'] = df_labeleddata['velocidad_viento'].apply(discretize_wind_speed)
    df_labeleddata['discretized_wave_height'] = df_labeleddata['altura_oleaje'].apply(discretize_wave_height)

    # Columnas a graficar
    columnas_graficas = ['discretized_wind_speed', 'discretized_wave_height', 'direccion_viento_card']
    # Figura 13: Relación valoración por perfil con algunas variables
    plot_perfiles_columnas(df_labeleddata['perfil_rider'].unique().tolist(),
                           columnas_graficas,
                           df_labeleddata)

    # 7.2.9 Normalización
    # 7.2.9.1 Simplificación del conjunto de datos
    columns_to_drop = ['spot', 'fecha', 'dir_card', 'direccion_viento_card', 'direccion_oleaje_card',
                       'mes', 'año', 'discretized_wind_speed', 'discretized_wave_height']
    df_labeleddata.drop(columns=columns_to_drop, inplace=True)

    # 7.2.9.2 Codificación one-hot id_perfil_rider
    df_labeleddata = pd.get_dummies(df_labeleddata, columns=['id_perfil_rider'], prefix=['perfil_rider'])
    df_labeleddata.rename(columns={
        'perfil_rider_1': 'Novel',
        'perfil_rider_2': 'Avanzado',
        'perfil_rider_3': 'Pro'
    }, inplace=True)
    df_labeleddata.drop(columns=['perfil_rider'], inplace=True)

    # 7.2.9.3 Normalizacion variables continuas
    variables_continuas = ['racha', 'altura_oleaje', 'periodo_medio_oleaje', 'periodo_pico_oleaje',
                           'velocidad_viento', 'velocidad_corriente', 'peso_rider',
                           'sail_size', 'wind_board_size', 'edad_rider']

    df_labeleddata = standardize_continuous_features(df_labeleddata, variables_continuas)

    # 7.2.9.4 Normalización variables circulares
    df_labeleddata['direccion_viento'] = norm_grados(df_labeleddata['direccion_viento'])
    df_labeleddata['dir'] = norm_grados(df_labeleddata['dir'], True)
    df_labeleddata['direccion_oleaje'] = norm_grados(df_labeleddata['direccion_oleaje'])
    df_labeleddata['direccion_corriente'] = norm_grados(df_labeleddata['direccion_corriente'])

    # 7.2.9.5 Tratamiento de valores NaN por la media
    imputer = SimpleImputer(strategy='mean')
    df_labeleddata = pd.DataFrame(imputer.fit_transform(df_labeleddata), columns=df_labeleddata.columns)

    # 7.2.10 Conjunto de datos final
    # Imprimir las primeras filas del DataFrame
    print("Primeras filas del DataFrame:")
    print(df_labeleddata.head())

    # Imprimir información del DataFrame
    print("\nInformación del DataFrame:")
    print(df_labeleddata.info())

    # Imprimir estadísticas descriptivas del DataFrame
    print("\nEstadísticas descriptivas del DataFrame:")
    print(df_labeleddata.describe())

    df_labeleddata.to_json('DATA/PCA/pca_input.json', orient='records', lines=True)
