-- Data Model to load data from sources

CREATE TABLE IF NOT EXISTS AEMET_ESTACION (
            latitud	NUMERIC,
            provincia	VARCHAR(255),
            altitud	VARCHAR(255),
            indicativo	VARCHAR(255),
            nombre	VARCHAR(255),
            indsinop	VARCHAR(255),
            longitud	NUMERIC,
            primary key (indicativo)
            );

COMMENT ON COLUMN AEMET_ESTACION.latitud IS 'Latitud de la estación';
COMMENT ON COLUMN AEMET_ESTACION.provincia IS 'Provincia donde reside la estación';
COMMENT ON COLUMN AEMET_ESTACION.indicativo IS 'Indicativo climatológico de la estación';
COMMENT ON COLUMN AEMET_ESTACION.altitud IS 'Altitud de la estación';
COMMENT ON COLUMN AEMET_ESTACION.nombre IS 'Ubicación de la estación';
COMMENT ON COLUMN AEMET_ESTACION.indsinop IS 'Indicativo sinóptico';
COMMENT ON COLUMN AEMET_ESTACION.longitud IS 'Longitud de la estación';

CREATE TABLE AEMET_HISTORICO (
    fecha TIMESTAMP NOT NULL,
    indicativo VARCHAR(255) REFERENCES AEMET_ESTACION(indicativo) ON DELETE CASCADE,
    nombre VARCHAR(255) NOT NULL,
    provincia VARCHAR(255) NOT NULL,
    altitud FLOAT NOT NULL,
    tmed FLOAT,
    prec FLOAT,
    tmin FLOAT,
    horatmin VARCHAR(5),
    tmax FLOAT,
    horatmax VARCHAR(5),
    dir FLOAT,
    velmedia FLOAT,
    racha FLOAT,
    horaracha VARCHAR(5),
    sol FLOAT,
    presmax FLOAT,
    horapresmax VARCHAR(5),
    presmin FLOAT,
    horapresmin VARCHAR(5),
    hrmedia FLOAT,
    hrmax FLOAT,
    horahrmax VARCHAR(5),
    hrmin FLOAT,
    horahrmin VARCHAR(5),
    PRIMARY KEY (fecha, indicativo) -- Clave primaria compuesta por fecha e indicativo
);

COMMENT ON COLUMN AEMET_HISTORICO.fecha IS 'Fecha del día (AAAA-MM-DD), convertida a TIMESTAMP';
COMMENT ON COLUMN AEMET_HISTORICO.indicativo IS 'Indicativo climatológico';
COMMENT ON COLUMN AEMET_HISTORICO.nombre IS 'Nombre (ubicación) de la estación';
COMMENT ON COLUMN AEMET_HISTORICO.provincia IS 'Provincia de la estación';
COMMENT ON COLUMN AEMET_HISTORICO.altitud IS 'Altitud de la estación en metros sobre el nivel del mar';
COMMENT ON COLUMN AEMET_HISTORICO.tmed IS 'Temperatura media diaria en °C';
COMMENT ON COLUMN AEMET_HISTORICO.prec IS 'Precipitación diaria de 07 a 07 en mm (Ip = inferior a 0,1 mm) (Acum = Precipitación acumulada)';
COMMENT ON COLUMN AEMET_HISTORICO.tmin IS 'Temperatura Mínima del día en °C';
COMMENT ON COLUMN AEMET_HISTORICO.horatmin IS 'Hora y minuto de la temperatura mínima en UTC';
COMMENT ON COLUMN AEMET_HISTORICO.tmax IS 'Temperatura Máxima del día en °C';
COMMENT ON COLUMN AEMET_HISTORICO.horatmax IS 'Hora y minuto de la temperatura máxima en UTC';
COMMENT ON COLUMN AEMET_HISTORICO.dir IS 'Dirección de la racha máxima en decenas de grado (99 = dirección variable) (88 = sin dato)';
COMMENT ON COLUMN AEMET_HISTORICO.velmedia IS 'Velocidad media del viento en m/s';
COMMENT ON COLUMN AEMET_HISTORICO.racha IS 'Racha máxima del viento en m/s';
COMMENT ON COLUMN AEMET_HISTORICO.horaracha IS 'Hora y minuto de la racha máxima en UTC';
COMMENT ON COLUMN AEMET_HISTORICO.sol IS 'Insolación en horas';
COMMENT ON COLUMN AEMET_HISTORICO.presmax IS 'Presión máxima al nivel de referencia de la estación en hPa';
COMMENT ON COLUMN AEMET_HISTORICO.horapresmax IS 'Hora de la presión máxima (redondeada a la hora entera más próxima) en UTC';
COMMENT ON COLUMN AEMET_HISTORICO.presmin IS 'Presión mínima al nivel de referencia de la estación en hPa';
COMMENT ON COLUMN AEMET_HISTORICO.horapresmin IS 'Hora de la presión mínima (redondeada a la hora entera más próxima) en UTC';
COMMENT ON COLUMN AEMET_HISTORICO.hrmedia IS 'Humedad relativa media diaria en %';
COMMENT ON COLUMN AEMET_HISTORICO.hrmax IS 'Humedad relativa máxima diaria en %';
COMMENT ON COLUMN AEMET_HISTORICO.horahrmax IS 'Hora de la humedad relativa máxima diaria en UTC';
COMMENT ON COLUMN AEMET_HISTORICO.hrmin IS 'Humedad relativa mínima diaria en %';
COMMENT ON COLUMN AEMET_HISTORICO.horahrmin IS 'Hora de la humedad relativa mínima diaria en UTC';


CREATE TABLE PUERTOS_PUNTO_MODELO (
    punto_modelo VARCHAR(50),
    latitud NUMERIC,
    longitud NUMERIC,
    PRIMARY KEY (punto_modelo)
);

CREATE TABLE PUERTOS_HISTORICO (
    fecha TIMESTAMP NOT NULL,
    altura_oleaje NUMERIC,
    periodo_medio_oleaje NUMERIC,
    periodo_pico_oleaje NUMERIC,
    direccion_oleaje NUMERIC,
    altura_mar_viento NUMERIC,
    direccion_mar_viento NUMERIC,
    altura_mar_fondo_1 NUMERIC,
    periodo_medio_mar_fondo_1 NUMERIC,
    direccion_mar_fondo_1 NUMERIC,
    altura_mar_fondo_2 NUMERIC,
    periodo_medio_mar_fondo_2 NUMERIC,
    direccion_mar_fondo_2 NUMERIC,
    punto_modelo VARCHAR(50) REFERENCES PUERTOS_PUNTO_MODELO(punto_modelo) ON DELETE CASCADE,
    velocidad_viento NUMERIC,
    direccion_viento NUMERIC,
    velocidad_corriente NUMERIC,
    direccion_corriente NUMERIC,
    temperatura_agua NUMERIC,
    salinidad NUMERIC,
    PRIMARY KEY (fecha, punto_modelo)
);

COMMENT ON COLUMN PUERTOS_HISTORICO.fecha IS 'Fecha (GMT)';
COMMENT ON COLUMN PUERTOS_HISTORICO.altura_oleaje IS 'Altura Signif. del Oleaje(m)';
COMMENT ON COLUMN PUERTOS_HISTORICO.periodo_medio_oleaje IS 'Periodo Medio(s)';
COMMENT ON COLUMN PUERTOS_HISTORICO.periodo_pico_oleaje IS 'Periodo de Pico(s)';
COMMENT ON COLUMN PUERTOS_HISTORICO.direccion_oleaje IS 'Direcc. Media de Proced.(0=N,90=E)';
COMMENT ON COLUMN PUERTOS_HISTORICO.altura_mar_viento IS 'Mar de viento:  Altura signif. Espectral(m)';
COMMENT ON COLUMN PUERTOS_HISTORICO.direccion_mar_viento IS 'Mar de viento: Direcc. Media de Proced.(0=N,90=E)';
COMMENT ON COLUMN PUERTOS_HISTORICO.altura_mar_fondo_1 IS 'Mar de fondo 1:  Altura signif. Espectral(m)';
COMMENT ON COLUMN PUERTOS_HISTORICO.periodo_medio_mar_fondo_1 IS 'Mar de fondo 1:  Periodo Medio Espectral(s)';
COMMENT ON COLUMN PUERTOS_HISTORICO.direccion_mar_fondo_1 IS 'Mar de fondo 1: Direcc. Media de Proced.(0=N,90=E)';
COMMENT ON COLUMN PUERTOS_HISTORICO.altura_mar_fondo_2 IS 'Mar de fondo 2:  Altura signif. Espectral(m)';
COMMENT ON COLUMN PUERTOS_HISTORICO.periodo_medio_mar_fondo_2 IS 'Mar de fondo 2:  Periodo Medio Espectral(s)';
COMMENT ON COLUMN PUERTOS_HISTORICO.direccion_mar_fondo_2 IS 'Mar de fondo 2: Direcc. Media de Proced.(0=N,90=E)';
COMMENT ON COLUMN PUERTOS_HISTORICO.punto_modelo IS 'PuntoModelo';
COMMENT ON COLUMN PUERTOS_HISTORICO.velocidad_viento IS 'Velocidad del viento(m/s)';
COMMENT ON COLUMN PUERTOS_HISTORICO.direccion_viento IS 'Direc. de proced. del Viento(0=N,90=E)';
COMMENT ON COLUMN PUERTOS_HISTORICO.velocidad_corriente IS 'Velocidad media de Corriente(m/s)';
COMMENT ON COLUMN PUERTOS_HISTORICO.direccion_corriente IS 'Dir. de prop. de la Corriente (0=N,90=E)';
COMMENT ON COLUMN PUERTOS_HISTORICO.temperatura_agua IS 'Temperatura del Agua(ºC)';
COMMENT ON COLUMN PUERTOS_HISTORICO.salinidad IS 'Salinidad(psu)';

CREATE TABLE IF NOT EXISTS WC_DEPORTE (
    ID SERIAL PRIMARY KEY ,
    DEPORTE VARCHAR(255)
);

CREATE TABLE IF NOT EXISTS WC_MODALIDAD (
    ID SERIAL PRIMARY KEY ,
    MODALIDAD VARCHAR(255)
);

CREATE TABLE IF NOT EXISTS WC_SPOT (
    id SERIAL PRIMARY KEY,
    nombre VARCHAR(255) NOT NULL,
    descripcion VARCHAR(2000),
    latitud FLOAT NOT NULL,
    longitud FLOAT NOT NULL
);

CREATE TABLE IF NOT EXISTS WC_SPOT_WC_MODALIDAD (
    id_spot INT REFERENCES WC_SPOT(id) ON DELETE CASCADE,
    id_modalidad INT REFERENCES WC_MODALIDAD(id) ON DELETE CASCADE,
    PRIMARY KEY (id_spot, id_modalidad)
);

CREATE TABLE IF NOT EXISTS WC_SPOT_AEMET_ESTACION (
    id_spot     INT REFERENCES WC_SPOT(id) ON DELETE CASCADE,
    id_estacion VARCHAR(255) REFERENCES AEMET_ESTACION(indicativo) ON DELETE CASCADE,
    PRIMARY KEY (id_estacion, id_spot)
);

CREATE TABLE IF NOT EXISTS WC_SPOT_PUERTOS_PUNTO_MODELO (
    id_spot     INT REFERENCES WC_SPOT(id) ON DELETE CASCADE,
    punto_modelo VARCHAR(50) REFERENCES PUERTOS_PUNTO_MODELO(punto_modelo) ON DELETE CASCADE,
    PRIMARY KEY (punto_modelo, id_spot)
);


-- INSERTS PUERTOS_PUNTO_MODELO
INSERT INTO PUERTOS_PUNTO_MODELO (punto_modelo,latitud,longitud) VALUES
     ('4033010',28.08,-15.75),
     ('4034011',28.17,-15.67),
     ('4035005',27.67,-15.58),
     ('4035011',28.17,-15.58),
     ('4036006',27.75,-15.50),
     ('4037007',27.83,-15.42),
     ('4038008',27.92,-15.33),
     ('421038045',28.05,-15.39);



-- INSERTS WC_DEPORTE
INSERT INTO WC_DEPORTE (DEPORTE)
VALUES
('Windsurf'),
('Kitesurf'),
('Wingfoil');


-- INSERTS WC_MODALIDAD
INSERT INTO WC_MODALIDAD (MODALIDAD) VALUES
('Olas'),
('Slalom'),
('Freeride'),
('Freestyle'),
('Big Air'),
('Surf-Freestyle'),
('Freefly-Slalom');

-- INSERTS WC_SPOT
INSERT INTO WC_SPOT (nombre, descripcion, latitud, longitud) VALUES
('Playa Pozo Izquierdo (El Arenal)', 'Pozo Izquierdo es uno de los spots de olas más conocidos a nivel mundial, por ser el escenario de una de las pruebas del circuito internacional...', 27.821239, -15.423346),
('Salinas de Pozo', NULL, 27.813116, -15.42223),
('Castillo del Romeral', 'Spot freestyle y olas', 27.794136, -15.472097),
('Bahía Feliz', NULL, 27.781702, -15.517148),
('Playa del Águila', NULL, 27.775931, -15.528585),
('Playa del Inglés', NULL, 27.741885, -15.568399),
('El Faro', NULL, 27.7352, -15.59741),
('Agaete', 'En la zona de Agaete es el otro punto de la Isla de Gran Canaria, donde más se acelera el viento. Se considera un spot de olas peligroso con viento del NE, donde rompe una derecha delante del espigón, muy pegado a las rocas. Con SW es on - shore para freeride. Es fácil romper material.', 28.103746, -15.712681),
('Gáldar, Bocabarranco', 'Spot de olas situado el NO de la Isla, con condiciones difíciles y peligrosas, tanto por la fuerza de las olas, las corrientes y viento inestables. La entrada y salida del agua es por rocas (riscos). Spot caracterizado por destrozar el material. Es una zona de Surfing con localismo. Es importante respetar a los surfistas.', 28.158433, -15.665174),
('San Andrés', 'Las costa de San Andrés es conocida por ser la zona de Surf del Norte de la Isla. Existen varios picos algo peligrosos para windsurf, por la fuerza de las olas, fuertes corrientes, viento inestable, y la entrada y salida del agua con rocas cortantes. Spot caracterizado por lo fácil que es romper material. Es importante respetar a los surfistas.', 28.146968, -15.559988),
('Playa La Restinga', 'Spot de olas peligroso, por las corrientes y el viento inestable. Zona de Kite donde se puede también navegar en windsurf.', 28.028679, -15.3910244),
('Playa Ojos de Garza', 'Spot de olas de roca algo más difícil que Pozo y Vargas. En la zona derecha de la playa hay un rompeolas muy peligroso, sobretodo con marea alta. Se puede tener problemas con las surfistas, es importante respetarlos.', 27.9494986, -15.3767923),
('El Burrero', 'Spot Mixto, con viento racheado en la orilla, debido al risco. Fuera de la playa viento limpio pero con mucho chopy. Espigo y rocas, peligrosas situadas a la derecha de la playa.', 27.907941, -15.386192),
('Erizos', 'Spot de olas difícil, con fondo de callados grandes y cortantes, erizos y viento racheado. Cueva muy peligrosa al final de la ola donde sueles terminar si te caes, fácil romper material.', 27.893090, -15.391145),
('Playa de Vargas', 'Spot de olas (NE, izquierdas) conocido por ser algo más side - shore que Pozo Izquierdo, siendo mejor para surfear. Funciona con las mismas condiciones de viento que Pozo Izquierdo, soplando para medio metro más de vela, entrando algo más de ola que en Pozo Izquierdo. También es conocida por ser la playa donde vive Philip Koster. Con viento del S – SW, (derechas, side off shore) es un spot perfecto para iniciación. Con marea baja es un spot fácil, pero con marea alta y olas hay un shorebrake fuerte que rompe encima de los callados.', 27.8896897, -15.3928967),
('Bahía de Formas (Cuevalaya)', 'Bahía de Formas es un spot de iniciación, slalom y freestyle. Es una playa amplia de arena y callados, perfecta para iniciarte al mundo del windsurf, gracias a sus condiciones de viento y agua plana. Únicamente tiene dos zonas situadas a los extremos de la playa donde hay que tener cuidado con las rocas.', 27.847972, -15.412712),
('Playa de las Burras', NULL, 27.765870, -15.552548);

-- INSERT WC_SPOT_WC_MODALIDAD
INSERT INTO WC_SPOT_WC_MODALIDAD (id_spot, id_modalidad) VALUES
(1, 1), (1, 2), (1, 3), (1, 4), (1, 5), (1, 6), (1, 7),
(2, 1), (2, 5), (2, 6),
(3, 1), (3, 4), (3, 6),
(4, 2), (4, 3), (4, 4), (4, 7),
(5, 2), (5, 3), (5, 4), (5, 7),
(6, 1), (6, 6),
(7, 1), (7, 6),
(8, 1), (8, 3), (8, 6),
(9, 1), (9, 6),
(10, 1), (10, 6),
(11, 1), (11, 6),
(12, 1), (12, 6),
(13, 1), (13, 3), (13, 6),
(14, 1), (14, 6),
(15, 1), (15, 3), (15, 4), (15, 5), (15, 6),
(16, 2), (16, 3), (16, 4), (16, 7),
(17, 1), (17, 3), (17, 6);

-- COMMIT DATA
COMMIT;
