import math


def distance_by_haversine(lat1, lon1, lat2, lon2):
    """
    Calcula la distancia en kilómetros entre dos puntos en la Tierra dados sus valores de latitud y longitud
    utilizando la fórmula de Haversine.

    Args:
    lat1 (float): Latitud del primer punto en grados.
    lon1 (float): Longitud del primer punto en grados.
    lat2 (float): Latitud del segundo punto en grados.
    lon2 (float): Longitud del segundo punto en grados.

    Returns:
    float: Distancia entre los dos puntos en kilómetros.
    """
    # Radio de la Tierra en kilómetros
    radio_tierra = 6371.0

    # Convertir grados a radianes
    lat1_radianes = math.radians(lat1)
    lon1_radianes = math.radians(lon1)
    lat2_radianes = math.radians(lat2)
    lon2_radianes = math.radians(lon2)

    # Diferencia de latitud y longitud
    dif_latitud = lat2_radianes - lat1_radianes
    dif_longitud = lon2_radianes - lon1_radianes

    # Fórmula de Haversine
    a = math.sin(dif_latitud / 2) ** 2 + math.cos(lat1_radianes) * math.cos(lat2_radianes) * math.sin(
        dif_longitud / 2) ** 2
    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))
    distancia = radio_tierra * c

    return distancia


def transform_to_grades(latitud, longitud):
    """
    Convierte coordenadas en formato de grados, minutos y dirección (N, S, E, W) a grados decimales.

    Args:
    latitud (str): Latitud en formato de grados, minutos y dirección (N, S).
    longitud (str): Longitud en formato de grados, minutos y dirección (E, W).

    Returns:
    tuple: Tupla con las coordenadas convertidas a grados decimales (latitud, longitud).
    """
    # Función interna para convertir una coordenada
    def convertir_coordenada(coordenada):
        direccion = coordenada[-1]  # Obtener la dirección (N, S, E, W)
        grados_minutos = coordenada[:-1]  # Obtener los grados y minutos
        grados = float(grados_minutos[:-4])
        minutos = float(grados_minutos[-4:-2])
        segundos = float(grados_minutos[-2:])
        grados_decimales = grados + minutos / 60 + segundos / 3600  # Convertir a grados decimales
        if direccion in ['S', 'W']:  # Ajustar la dirección (N, S, E, W)
            grados_decimales *= -1
        return grados_decimales

    # Convertir la latitud y la longitud
    latitud_grados = convertir_coordenada(latitud)
    longitud_grados = convertir_coordenada(longitud)

    return latitud_grados, longitud_grados  # Devolver las coordenadas en grados decimales

