import seaborn as sns
import matplotlib.pyplot as plt
from sklearn.utils import resample
import numpy as np
from sklearn.preprocessing import StandardScaler
import pandas as pd


def list_to_sqlstrlist(itemlist):
    # Convertir una lista en una cadena separada por comas
    strlst = ''
    for item in itemlist:
        strlst += "'" + item + "',"
    strlst = strlst[:-1]
    return strlst


def plot_corr_matrix(corr_matrix):
    # Gráfico
    # Configurar el tamaño del plot
    plt.figure(figsize=(14, 12))
    # Graficar la matriz de correlación usando un mapa de calor de Seaborn
    heatmap = sns.heatmap(corr_matrix, annot=True, cmap='coolwarm', fmt=".2f")
    # Rotar los nombres de las columnas en el eje x
    heatmap.set_xticklabels(heatmap.get_xticklabels(), rotation=45, horizontalalignment='right')
    # Añadir título
    plt.title('Matriz de correlación')
    # Mostrar el gráfico
    plt.show()


def tratar_nulos(dataframe, umbral=95):

    print("Tratamiento de valores nulos")
    # Calcular el porcentaje de nulos para cada columna
    total_filas = len(dataframe)
    porcentaje_nulos = round((dataframe.isnull().sum() / total_filas) * 100, 2)

    resultados = []  # Lista para almacenar los resultados

    # Iterar sobre las columnas y calcular el porcentaje de valores nulos
    for columna, porcentaje in porcentaje_nulos.items():
        eliminada = 'Sí' if porcentaje > umbral else 'No'  # Determinar si la columna será eliminada
        resultados.append([columna, porcentaje, eliminada])  # Agregar los resultados a la lista
        if porcentaje > umbral:
            # Descartar la característica
            dataframe = dataframe.drop(columna, axis=1)

    # Crear un DataFrame con los resultados
    resultados_df = pd.DataFrame(resultados, columns=['Variable', 'Nulos', 'Eliminada'])

    # Imprimir el DataFrame
    print(resultados_df)
    print()
    return dataframe


def tratar_varianzas(dataframe, umbral=0.01):

    print("Descartando características con una varianza inferior a 0.01")
    # Seleccionar solo las columnas numéricas
    columnas_numericas = dataframe.select_dtypes(include=['int', 'float']).columns

    # Calcular las varianzas de las columnas numéricas
    varianzas = round(dataframe[columnas_numericas].var(), 2)

    resultados = []  # Lista para almacenar los resultados

    # Imprimir las varianzas de cada columna y eliminar las que tienen baja varianza
    for columna, varianza in varianzas.items():
        eliminada = 'Sí' if varianza < umbral else 'No'  # Determinar si la columna será eliminada
        resultados.append([columna, varianza, eliminada])  # Agregar los resultados a la lista
        if varianza < umbral:
            del dataframe[columna]

    # Crear un DataFrame con los resultados
    resultados_df = pd.DataFrame(resultados, columns=['Variable', 'Varianza', 'Eliminada'])

    # Imprimir el DataFrame
    print(resultados_df)
    print()
    return dataframe


def exploracion_univariada(dataframe):
    # Seleccionar solo las columnas numéricas
    columnas_numericas = dataframe.select_dtypes(include=['int', 'float']).columns

    # Calcular el número de filas necesarias
    # para acomodar todos los gráficos en una matriz de 3 columnas
    num_columnas = len(columnas_numericas)
    num_filas = (num_columnas + 2) // 3  # Redondeo hacia arriba

    # Configuración de estilo de Seaborn
    sns.set(style="whitegrid")

    # Crear una figura y una matriz de subgráficos
    fig, axes = plt.subplots(num_filas, 3, figsize=(12, 5 * num_filas))
    fig.subplots_adjust(hspace=0.2, wspace=0.3)  # Espacio vertical entre los subgráficos

    # Iterar sobre las columnas numéricas y generar histogramas en cada subgráfico
    for i, columna in enumerate(columnas_numericas):
        fila = i // 3
        columna_grafico = i % 3

        # Gráfico de distribución con Seaborn
        sns.histplot(dataframe[columna], ax=axes[fila, columna_grafico], kde=True, color='blue',
                     line_kws={'color': 'red'})
        axes[fila, columna_grafico].set_title(f'Distribución de {columna}', fontsize=12)
        axes[fila, columna_grafico].set_xlabel(None, fontsize=10)
        axes[fila, columna_grafico].set_ylabel(None, fontsize=10)
        axes[fila, columna_grafico].tick_params(axis='both', which='major', labelsize=8)

    # Ocultar los subgráficos vacíos si el número de columnas no es divisible por 3
    for i in range(num_columnas, num_filas * 3):
        fila = i // 3
        columna_grafico = i % 3
        axes[fila, columna_grafico].axis('off')

    plt.show()


def num_tabular_exploracion_univariada(dataframe):
    # Seleccionar solo las columnas no numéricas
    columnas_no_numericas = dataframe.select_dtypes(include=['int', 'float']).columns

    # Iterar sobre las columnas no numéricas y tabular los datos según el tipo de datos
    for columna in columnas_no_numericas:
        # Imprimir el nombre de la columna
        print(f'Distribución de {columna}:\n')

        # Conteo de valores únicos y tabulación para columnas categóricas
        if len(dataframe[columna].unique()) < 10:  # Consideramos categorías si hay menos de 10 valores únicos
            tabla_frecuencia = dataframe[columna].value_counts().reset_index()
            tabla_frecuencia.columns = [columna, 'Frecuencia']
            print(tabla_frecuencia)
        else:  # Para variables categóricas con muchos valores únicos, simplemente mostramos los primeros 10 valores más frecuentes
            top_10_valores = dataframe[columna].value_counts().head(10).reset_index()
            top_10_valores.columns = [columna, 'Frecuencia']
            print(top_10_valores)

        print('\n')


def tabular_exploracion_univariada(dataframe):
    # Seleccionar solo las columnas no numéricas
    columnas_no_numericas = dataframe.select_dtypes(exclude=['int', 'float']).columns

    # Iterar sobre las columnas no numéricas y tabular los datos según el tipo de datos
    for columna in columnas_no_numericas:
        # Imprimir el nombre de la columna
        print(f'Distribución de {columna}:\n')

        # Conteo de valores únicos y tabulación para columnas categóricas
        if len(dataframe[columna].unique()) < 10:  # Consideramos categorías si hay menos de 10 valores únicos
            tabla_frecuencia = dataframe[columna].value_counts().reset_index()
            tabla_frecuencia.columns = [columna, 'Frecuencia']
            print(tabla_frecuencia)
        else:  # Para variables categóricas con muchos valores únicos, simplemente mostramos los primeros 10 valores más frecuentes
            top_10_valores = dataframe[columna].value_counts().head(10).reset_index()
            top_10_valores.columns = [columna, 'Frecuencia']
            print(top_10_valores)

        print('\n')


def str_exploracion_univariada(dataframe):

    # Seleccionar solo las columnas no numéricas
    columnas_no_numericas = dataframe.select_dtypes(exclude=['int', 'float']).columns

    # Calcular el número de filas necesarias para acomodar todos los gráficos en una matriz de 2 columnas
    num_columnas_no_numericas = len(columnas_no_numericas)
    num_filas_no_numericas = (num_columnas_no_numericas + 1) // 2  # Redondeo hacia arriba

    # Configuración de estilo de Seaborn
    sns.set(style="whitegrid")

    # Crear una figura y una matriz de subgráficos
    fig, axes = plt.subplots(num_filas_no_numericas, 2, figsize=(12, num_filas_no_numericas * 6), squeeze=False,
                             gridspec_kw=dict(left=0.05, right=0.95, top=0.95, bottom=0.05, wspace=0.2, hspace=0.5))

    # Iterar sobre las columnas no numéricas y generar gráficos según el tipo de datos
    for i, columna in enumerate(columnas_no_numericas):
        fila = i // 2
        columna_grafico = i % 2

        # Histograma con KDE para columnas categóricas
        sns.histplot(data=dataframe, x=columna, stat="density", kde=True, color='blue', ax=axes[fila, columna_grafico])

        axes[fila, columna_grafico].set_title(f'Distribución de {columna}', fontsize=12)
        axes[fila, columna_grafico].set_xlabel(None, fontsize=10)
        axes[fila, columna_grafico].set_ylabel(None, fontsize=10)
        axes[fila, columna_grafico].tick_params(axis='both', which='major', labelsize=8)
        axes[fila, columna_grafico].tick_params(axis='x', rotation=90)

    # Ocultar los subgráficos vacíos si el número de columnas no es divisible por 2
    for i in range(num_columnas_no_numericas, num_filas_no_numericas * 2):
        fila = i // 2
        columna_grafico = i % 2
        axes[fila, columna_grafico].axis('off')

    plt.tight_layout()
    plt.show()


def visualizacion_multivariada(dataframe):
    # Seleccionar solo las columnas numéricas
    columnas_numericas = dataframe.select_dtypes(include=['int', 'float']).columns

    # Calcular el número de filas necesarias
    # para acomodar todos los gráficos en una matriz de 3 columnas
    num_columnas = len(columnas_numericas)
    num_filas = (num_columnas + 2) // 3  # Redondeo hacia arriba

    # Crear una figura y una matriz de subgráficos
    fig, axes = plt.subplots(num_filas, 3, figsize=(15, 5 * num_filas))
    fig.subplots_adjust(hspace=0.5)  # Espacio vertical entre los subgráficos

    # Iterar sobre las columnas numéricas y generar la matriz de dispersión
    for i, columna in enumerate(columnas_numericas):
        fila = i // 3
        columna_grafico = i % 3

        sns.kdeplot(data=dataframe[columna], ax=axes[fila, columna_grafico], fill=True, alpha=0.5)
        axes[fila, columna_grafico].set_title(f'Distribución de {columna}')
        axes[fila, columna_grafico].set_xlabel('Valor')
        axes[fila, columna_grafico].set_ylabel('Densidad')

    # Ocultar los subgráficos vacíos si el número de columnas no es divisible por 3
    for i in range(num_columnas, num_filas * 3):
        fila = i // 3
        columna_grafico = i % 3
        axes[fila, columna_grafico].axis('off')

    plt.show()


def proporcion_clases(df, columnas):
    for columna in columnas:
        proporciones = {}

        # Filtrar los valores nulos en la columna
        df_filtrado = df.dropna(subset=[columna])

        # Calcular la proporción de cada clase en la columna
        conteo_clases = df_filtrado[columna].value_counts()
        total_instancias = len(df_filtrado)
        proporciones[columna] = round(100 * (conteo_clases / total_instancias), 2)

        # Crear un DataFrame para la columna actual y mostrarlo
        df_prop = pd.DataFrame(proporciones)
        df_prop.columns = ['Proporción (%)']
        print(f"\nProporciones de clases en la columna '{columna}':")
        print(df_prop)


def undersample_variable(df, variable, random_state=None):
    # Conteo de clases
    conteo_clases = df[variable].value_counts()

    # Determinar la clase con la menor cantidad de muestras
    min_muestras = min(conteo_clases)

    # Realizar undersampling para igualar el número de muestras en todas las clases
    muestras_undersampled = []
    for clase, cantidad in conteo_clases.items():
        df_clase = df[df[variable] == clase]
        undersampled_clase = resample(df_clase, replace=False, n_samples=min_muestras,
                                      random_state=random_state)
        muestras_undersampled.append(undersampled_clase)

    # Combinar los resultados en un nuevo DataFrame
    df_undersampled = pd.concat(muestras_undersampled)

    return df_undersampled


def oversample_variable(df, variable, random_state=None):
    # Conteo de clases
    conteo_clases = df[variable].value_counts()

    # Determinar la clase con la mayor cantidad de muestras
    max_muestras = max(conteo_clases)

    # Realizar oversampling para igualar el número de muestras en todas las clases
    muestras_oversampled = []
    for clase, cantidad in conteo_clases.items():
        df_clase = df[df[variable] == clase]
        if cantidad < max_muestras:
            oversampled_clase = resample(df_clase, replace=True, n_samples=max_muestras,
                                         random_state=random_state)
            muestras_oversampled.append(oversampled_clase)
        else:
            muestras_oversampled.append(df_clase)

    # Combinar los resultados en un nuevo DataFrame
    df_oversampled = pd.concat(muestras_oversampled)

    return df_oversampled


def grados_a_cardinal(grados, racha=False):
    if grados is None:
        return grados
    if pd.isna(grados):
        return None
    if racha:
        grados = grados * 10
        if grados == 990:
            return 'VAR'
    grados = (grados + 360) % 360
    direcciones = ['N', 'NE', 'E', 'SE', 'S', 'SO', 'O', 'NO']
    indice = int((grados + 22.5) // 45) % 8

    return direcciones[indice]


def plot_versus(df, columna_x, columna_y, title=None, orderby=None):
    if orderby == 'y':
        df = df.sort_values(by=columna_y, ascending=False)
    else:
        df = df.sort_values(by=columna_x, ascending=False)
    sns.set(style="whitegrid")
    plt.figure(figsize=(8, 6))
    sns.barplot(x=columna_x, y=columna_y, data=df, palette="Blues")
    if title is not None:
        plt.title(title)
    else:
        plt.title(f'Relación entre {columna_x} y {columna_y}')
    plt.xlabel(columna_x)
    plt.ylabel(columna_y)
    plt.grid(True)
    plt.show()


# Función de discretización para la velocidad del viento
def discretize_wind_speed(speed):

    if speed <= 15:
        return '0-15'
    elif speed <= 20:
        return '16-20'
    elif speed <= 25:
        return '21-25'
    elif speed <= 30:
        return '26-30'
    else:
        return '>30'


def discretize_wave_height(height):

    if height <= 0.5:
        return '>0.5'
    elif height <= 1:
        return '0.5-1.0'
    elif height <= 1.5:
        return '1.0-1.5'
    elif height <= 2:
        return '1.5-2.0'
    else:
        return '>2.0'


def plot_perfiles_columnas(perfiles, columnas_graficas, df_labeleddata):
    # Definir el tamaño de la matriz de subgráficos
    filas = len(perfiles)
    columnas = len(columnas_graficas)

    # Crear la matriz de subgráficos
    fig, axs = plt.subplots(filas, columnas, figsize=(15, 10))

    # Bucle para generar los gráficos para cada perfil de rider
    for i, perfil in enumerate(perfiles):
        for j, columna in enumerate(columnas_graficas):
            # Filtrar el DataFrame por el perfil de rider y la columna actual
            df_filtrado = df_labeleddata[df_labeleddata['perfil_rider'] == perfil]

            # Calcular la media de 'valoracion' para cada valor único en la columna actual
            valores_unicos = df_filtrado[columna].unique()
            medias = [df_filtrado[df_filtrado[columna] == valor]['valoracion'].mean() for valor in valores_unicos]

            # Ordenar los valores únicos
            valores_unicos_sorted = np.sort(valores_unicos)

            # Convertir valores únicos a una lista estándar de Python
            valores_unicos_list = valores_unicos_sorted.tolist()

            # Obtener las medias correspondientes a los valores ordenados
            medias_sorted = [medias[valores_unicos_list.index(valor)] for valor in valores_unicos_sorted]

            # Graficar en el subgráfico correspondiente
            # sns.barplot(x=columna, y='valoracion', data=df_filtrado, order=valores_unicos_sorted, ax=axs[i, j],
            #            palette="Blues")
            sns.barplot(x=columna, y='valoracion', hue=columna, data=df_filtrado, order=valores_unicos_sorted,
                        ax=axs[i, j], palette="Blues", legend=False)

            # Añadir etiquetas y título al gráfico
            axs[i, j].set_xlabel(columna)
            axs[i, j].set_ylabel('Valoración Media')
            axs[i, j].set_title(f'Perfil Rider: {perfil}\n {columna} vs Valoración Media')

    # Ajustar el diseño de los subgráficos
    plt.tight_layout()

    # Mostrar los gráficos
    plt.show()


def standardize_continuous_features(df, columnsToStandardize):
    # Inicializar el escalador
    scaler = StandardScaler()

    # Aplicar la estandarización a las características continuas
    df_standardized = df.copy()
    df_standardized[columnsToStandardize] = scaler.fit_transform(df[columnsToStandardize])

    # Reemplazar las columnas originales con las columnas estandarizadas
    df[columnsToStandardize] = df_standardized[columnsToStandardize]

    return df


def norm_grados(grados, racha=False):
    if racha:
        grados = grados * 10
        grados[grados == 990] = None

    return np.cos(2 * np.pi * grados / 360)
