import os
import pickle
import time
import warnings
from datetime import datetime

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
import xgboost as xgb
from joblib import dump, load
from sklearn.ensemble import RandomForestRegressor
from sklearn.metrics import mean_squared_error, mean_absolute_error, r2_score
from sklearn.model_selection import KFold, train_test_split, GridSearchCV, RandomizedSearchCV
from sklearn.preprocessing import StandardScaler

from constants import FOLDS, RANDOM_STATE, BATCH_SIZE, \
    EPOCHS, VERBOSE, OPTIMIZER, LOSS, MODELS_DIR, SHUFFLE, MODELS_CONT

os.environ['TF_ENABLE_ONEDNN_OPTS'] = '0'
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'

import tensorflow as tf


def print_metrics(df, target_col, option='', _return=True):
    models = MODELS_CONT
    data = {'Model': [], 'MSE': [], 'MAE': [], 'R2': [], 'TIME ELAPSED': []}
    X = df.drop(target_col, axis=1)
    y = df[target_col]
    kfold_validation = KFold(FOLDS, shuffle=SHUFFLE, random_state=RANDOM_STATE)

    for model_name, model in models.items():
        inicio = datetime.now()
        mse_scores = []
        mae_scores = []
        r2_scores = []

        for train_index, test_index in kfold_validation.split(X):
            X_train, X_test = X.iloc[train_index], X.iloc[test_index]
            y_train, y_test = y.iloc[train_index], y.iloc[test_index]

            # Entrenar el modelo
            model.fit(X_train, y_train)

            # Predecir con el conjunto de prueba
            y_pred = model.predict(X_test)

            # Calcular y almacenar las métricas
            mse_scores.append(mean_squared_error(y_test, y_pred))
            mae_scores.append(mean_absolute_error(y_test, y_pred))
            r2_scores.append(r2_score(y_test, y_pred))

        # Calcular el promedio de cada métrica
        cross_mse = sum(mse_scores) / len(mse_scores)
        cross_mae = sum(mae_scores) / len(mae_scores)
        cross_r2 = sum(r2_scores) / len(r2_scores)

        fin = datetime.now()
        time_elapsed = fin - inicio

        data['Model'].append(model_name)
        data['MSE'].append(cross_mse)
        data['MAE'].append(cross_mae)
        data['R2'].append(cross_r2)
        data['TIME ELAPSED'].append(time_elapsed.total_seconds())

    inicio = datetime.now()
    nnmse, nnmae, nnr2 = train_neural_network(df, target_col, kfold_validation, option)
    fin = datetime.now()
    time_elapsed = fin - inicio

    data['Model'].append("Neural Network")
    data['MSE'].append(nnmse)
    data['MAE'].append(nnmae)
    data['R2'].append(nnr2)
    data['TIME ELAPSED'].append(time_elapsed.total_seconds())

    df_metrics = pd.DataFrame(data)
    # Establecer la opción de pandas para mostrar todas las columnas
    pd.set_option('display.max_columns', None)
    if option == '':
        print('\n***** ', target_col, ' *****')
    else:
        print('\n***** ', option[:-1] + ": ", target_col, ' *****')
    print(df_metrics.to_string(index=False))
    if _return:
        return df_metrics


def plot_model_comparison_dual(df1, df2, title=''):
    # Crear un nuevo DataFrame combinando los datos de ambos DataFrames
    combined_df = pd.concat([df1, df2], keys=['Opción 1', 'Opción 2'])

    fig, axes = plt.subplots(4, 1, figsize=(9, 12), sharex=True)

    sns.barplot(ax=axes[0], data=combined_df, x='Model', y='MSE', hue=combined_df.index.get_level_values(0))
    axes[0].set_ylabel('MSE')

    sns.barplot(ax=axes[1], data=combined_df, x='Model', y='MAE', hue=combined_df.index.get_level_values(0))
    axes[1].set_ylabel('MAE')

    sns.barplot(ax=axes[2], data=combined_df, x='Model', y='R2', hue=combined_df.index.get_level_values(0))
    axes[2].set_ylabel('R2')

    sns.barplot(ax=axes[3], data=combined_df, x='Model', y='TIME ELAPSED', hue=combined_df.index.get_level_values(0))
    axes[3].set_ylabel('TIME ELAPSED')

    # Añadir título
    plt.suptitle(title, fontsize=16)

    # Rotar etiquetas del eje x en el último subplot
    plt.setp(axes[-1].xaxis.get_majorticklabels(), rotation=45, ha='right')

    # Mostrar la gráfica
    plt.tight_layout()

    for ax in axes:
        ax.legend(loc='upper right')
        for bar in ax.patches:
            # Obtener la posición x y y de la barra
            x = bar.get_x() + bar.get_width() / 2
            y = bar.get_height()
            ly = bar.get_y()
            # Añadir el valor numérico centrado en la barra
            if y != 0:
                ax.text(x, ly, f'{y:.3f}', ha='center', va='bottom', rotation=90)

    plt.show()


def train_neural_network(df, target_col, kf, option='', save=False):
    X = df.drop(columns=[target_col]).values
    y = df[target_col].values

    # Normalización de características
    scaler = StandardScaler()
    X_scaled = scaler.fit_transform(X)
    if save:
        dump(scaler, MODELS_DIR + option + target_col + "_" + 'NeuralNetwork.joblib')
    # Inicializar el modelo de red neuronal
    model = tf.keras.Sequential([
        tf.keras.layers.Dense(64, activation='relu'),
        tf.keras.layers.Dense(32, activation='relu'),
        tf.keras.layers.Dense(1)  # Linear activation for regression
    ])

    # Compilar el modelo
    model.compile(optimizer=OPTIMIZER, loss=LOSS)

    # Inicializar listas para almacenar métricas
    mse_scores = []
    r2_scores = []
    mae_scores = []

    for train_index, test_index in kf.split(X_scaled):
        X_train, X_test = X_scaled[train_index], X_scaled[test_index]
        y_train, y_test = y[train_index], y[test_index]

        # Entrenar el modelo
        model.fit(X_train, y_train, epochs=EPOCHS, batch_size=BATCH_SIZE, verbose=VERBOSE)

        # Evaluar el modelo
        y_pred = model.predict(X_test)
        mse = mean_squared_error(y_test, y_pred)
        mse_scores.append(mse)

        r2 = r2_score(y_test, y_pred)
        r2_scores.append(r2)

        mae = mean_absolute_error(y_test, y_pred)
        mae_scores.append(mae)

    # Calcular las métricas promedio
    mean_mse = np.mean(mse_scores)
    mean_r2 = np.mean(r2_scores)
    mean_mae = np.mean(mae_scores)

    # Devolver el modelo entrenado
    return mean_mse, mean_mae, mean_r2


def loadmodel(modelfile):
    if modelfile.endswith('.keras'):
        return tf.keras.models.load_model(MODELS_DIR + modelfile)
    return pickle.load(open(MODELS_DIR + modelfile, 'rb'))


def loadscaler(scalerfile):
    return load(MODELS_DIR + scalerfile)


def params_search(version, df, target_column, params, model_type='randomforest', search_type='random', folds=5):
    start_time_val = time.time()
    # Dividir los datos en características (X) y objetivo (y)
    X = df.drop(columns=[target_column])
    y = df[target_column]

    if model_type == 'randomforest':
        model = RandomForestRegressor()
    elif model_type == 'xgbregressor':
        model = xgb.XGBRegressor()
    else:
        raise ValueError("El tipo de modelo debe ser 'randomforest' o 'xgbregressor'.")

    # Configurar el tipo de búsqueda
    if search_type == 'random':
        search = RandomizedSearchCV(estimator=model, param_distributions=params, cv=folds,
                                    scoring='neg_mean_squared_error', n_jobs=-1)
    elif search_type == 'grid':
        search = GridSearchCV(estimator=model, param_grid=params, cv=folds,
                              scoring='neg_mean_squared_error', n_jobs=-1)
    else:
        raise ValueError("El tipo de búsqueda debe ser 'random' o 'grid'.")

    # Realizar la búsqueda en los datos
    warnings.filterwarnings("ignore")
    search.fit(X, y)

    # Obtener los mejores hiperparámetros
    best_params = search.best_params_

    # Entrenar el modelo final con los mejores hiperparámetros
    model.set_params(**best_params)
    model.fit(X, y)

    # Predecir en los datos de entrenamiento
    y_pred = model.predict(X)

    # Calcular métricas de evaluación
    mse = round(mean_squared_error(y, y_pred), 6)
    mae = round(mean_absolute_error(y, y_pred), 6)
    r2 = round(r2_score(y, y_pred), 6)
    print("********************************************************************************************************")
    print("")
    print("VARIABLE OBJETIVO:", target_column, "|", "MODELO:", model_type, "|", "BUSQUEDA TIPO:", search_type)
    print("")
    print("Mejores hiperparámetros:")
    # Imprimir cada parámetro en una línea
    for key, value in best_params.items():
        print(f"{key}: {value}")
    print("")
    print("Informe de métricas:")
    print("MSE:", mse)
    print("MAE:", mae)
    print("R2:", r2)
    print("")
    print("Búsqueda finalizada en",
          segundos_a_tiempo(time.time() - start_time_val))
    print("")
    print("********************************************************************************************************")
    # Guardo el modelo en el disco
    with open(MODELS_DIR + version + search_type + "_" + model_type + "_" + target_column + ".pkl", "wb") as file:
        pickle.dump(model, file)
    return best_params


def segundos_a_tiempo(segundos):
    horas = segundos // 3600
    minutos = (segundos % 3600) // 60
    segundos = segundos % 60

    tiempo_str = "{:02}:{:02}:{:02}".format(int(horas), int(minutos), int(segundos))
    return tiempo_str


def obtener_valoracion(valoracion):
    if isinstance(valoracion, str):
        if valoracion == "Ni me tiro":
            return 1
        elif valoracion == "Me mojé, un rato":
            return 2
        elif valoracion == "Buen baño":
            return 3
        elif valoracion == "Bañazo!":
            return 4
        elif valoracion == "Para Pro's":
            return 5
        else:
            return 99
    elif isinstance(valoracion, int) or isinstance(valoracion, float):
        if 1 <= valoracion <= 5:
            if 1 <= valoracion < 2:
                return "Ni me tiro"
            elif 2 <= valoracion < 3:
                return "Me mojé, un rato"
            elif 3 <= valoracion < 4:
                return "Buen baño"
            elif 4 <= valoracion < 5:
                return "Bañazo!"
            elif valoracion == 5:
                return "Para Pro's"
        else:
            return "Valoración fuera de rango"
    else:
        print("El tipo es", type(valoracion))
        return "El tipo debe ser 'int' o 'str'"


def windcaddyalg(version, peso, vel_viento, edad, racha, periodo):
    warnings.filterwarnings("ignore")
    val_data = np.array([peso, vel_viento, edad, racha, periodo]).reshape(1, -1)

    if version == "v1_":
        val_model = loadmodel('v1_grid_randomforest_valoracion.pkl')
        sail_model = loadmodel('v1_grid_xgbregressor_sail_size.pkl')
        wbs_model = loadmodel('v1_grid_randomforest_wind_board_size.pkl')
    elif version == "v2_":
        val_model = loadmodel('v2_v3_Random Forest_valoracion.pkl')
        sail_model = loadmodel('v2_op1_XGBRegressor_sail_size.pkl')
        wbs_model = loadmodel('v2_op1_Random Forest_wind_board_size.pkl')
    elif version == "v3_":
        val_model = loadmodel('v2_v3_Random Forest_valoracion.pkl')
        sail_model = loadmodel('v3_op2_XGBRegressor_sail_size.pkl')
        wbs_model = loadmodel('v3_op2_Random Forest_wind_board_size.pkl')
    else:
        print("Debes pasar la versión del algoritmo")
        return None, None, None

    valoracion = val_model.predict(val_data)
    if version == "v2_":
        valoracion_value = valoracion[0]
        if valoracion_value == 1:
            return 1, None, None
        else:
            sail_size = sail_model.predict(val_data)
            wbs_size = wbs_model.predict(val_data)
    elif version == "v3_":
        valoracion_value = valoracion[0]
        if valoracion_value == 1:
            return 1, None, None
        else:
            sail_data = np.hstack((val_data,
                                   valoracion_value.reshape(-1, 1)))
            sail_size = sail_model.predict(sail_data)
            wbs_data = np.hstack((val_data,
                                  sail_size.reshape(-1, 1),
                                  valoracion_value.reshape(-1, 1)))
            wbs_size = wbs_model.predict(wbs_data)
    else:
        valoracion_value = obtener_valoracion(valoracion.item())
        if valoracion_value == "Ni me tiro":
            return 1, None, None
        else:
            valoracion_value = obtener_valoracion(valoracion_value)

        sail_data = np.hstack((val_data, valoracion.reshape(-1, 1)))
        sail_size = sail_model.predict(sail_data)
        wbs_data = np.hstack((np.hstack((val_data, sail_size.reshape(-1, 1))), valoracion.reshape(-1, 1)))
        wbs_size = wbs_model.predict(wbs_data)

    sail_size_value = sail_size[0].round(decimals=1)
    wbs_value = wbs_size[0].round(decimals=0)

    return valoracion_value, sail_size_value, wbs_value


def do_prep_train_test_data():
    df = pd.read_json('DATA/FIII_DATASET/f3_dataset.json', orient='records', lines=True)
    df.dropna(inplace=True)
    train_df_val, test_df_val = train_test_split(df, test_size=0.2, random_state=RANDOM_STATE,
                                                 stratify=df["valoracion"])
    train_df_sail, test_df_sail = train_test_split(df, test_size=0.2, random_state=RANDOM_STATE,
                                                   stratify=df["sail_size"])
    train_df_wbs, test_df_wbs = train_test_split(df, test_size=0.2, random_state=RANDOM_STATE,
                                                 stratify=df["wind_board_size"])

    test_df = pd.concat([test_df_val, test_df_sail, test_df_wbs]).drop_duplicates()
    train_df_val.to_csv('DATA/WC_TRAIN_TEST_DATA/train_df_val.csv', index=False)
    train_df_sail.to_csv('DATA/WC_TRAIN_TEST_DATA/train_df_sail.csv', index=False)
    train_df_wbs.to_csv('DATA/WC_TRAIN_TEST_DATA/train_df_wbs.csv', index=False)
    test_df.to_csv("DATA/WC_TRAIN_TEST_DATA/test_df.csv", index=False)


#  return train_df_val, train_df_sail, train_df_wbs


def do_prepdfs(option, new=False):
    if new:
        do_prep_train_test_data()

    df_valoracion = pd.read_csv("DATA/WC_TRAIN_TEST_DATA/train_df_val.csv")
    df_sail_size = pd.read_csv("DATA/WC_TRAIN_TEST_DATA/train_df_sail.csv")
    df_wind_board_size = pd.read_csv("DATA/WC_TRAIN_TEST_DATA/train_df_wbs.csv")

    df_valoracion = df_valoracion.drop(['wind_board_size', 'sail_size'], axis=1)
    # Evaluación Opción 1
    if option == 1:
        df_sail_size = df_sail_size.drop(['wind_board_size', 'valoracion'], axis=1).dropna(subset=['sail_size'])
        df_wind_board_size = df_wind_board_size.drop(['sail_size', 'valoracion'], axis=1).dropna(
            subset=['wind_board_size'])
    elif option == 2:
        df_sail_size = df_sail_size.drop(['wind_board_size'], axis=1).dropna(subset=['sail_size'])
        df_wind_board_size = df_wind_board_size.dropna(subset=['wind_board_size'])
    else:
        return "Debe seleccionar una opcion valida"
    return df_valoracion, df_sail_size, df_wind_board_size
