import seaborn as sns
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from sklearn.decomposition import PCA


def plot_pca_components(pca, columns, minxlim=-1, maxxlim=1, minylim=-1, maxylim=1):
    # Configurar el estilo de Seaborn
    sns.set(style='whitegrid')

    # Crear el gráfico
    plt.figure(figsize=(10, 6))
    palette = sns.color_palette('tab20', n_colors=len(columns))  # Usar tantos colores como columnas
    colors = [palette[i] for i in range(len(columns))]
    legend_lines = []

    for i, var in enumerate(columns):
        color = colors[i]
        line = plt.arrow(0, 0, pca.components_[0, i], pca.components_[1, i],
                         color=color, width=0.0001,
                         head_width=0.01, head_length=0.03,
                         label=var)
        legend_lines.append(line)

    plt.legend(handles=legend_lines, loc='upper left', bbox_to_anchor=(1.05, 1), borderaxespad=0.)
    plt.xlim(minxlim, maxxlim)
    plt.ylim(minylim, maxylim)
    plt.title('PCA - Variables', fontsize=16)
    plt.xlabel('PC1', fontsize=14)
    plt.ylabel('PC2', fontsize=14)
    plt.grid(True)
    plt.tight_layout()
    plt.show()


def plot_loadings(eigen_vectors, num_components, columns):
    loadings_df = pd.DataFrame(eigen_vectors[:num_components], columns=columns).round(2)
    #print(f"Carga de cada variable en los {num_components} componentes principales:")
    plt.figure(figsize=(14, 14))
    heat_graph = sns.heatmap(loadings_df, annot=True, cmap='coolwarm',
                             yticklabels=[f"PC {i + 1}" for i in range(num_components)])
    heat_graph.set_title("Carga de las variables en los componentes principales",
                         fontdict={'fontsize': 18, 'fontfamily': 'Arial'})
    heat_graph.set_xticklabels(heat_graph.get_xticklabels(), rotation=45, ha='right', fontsize=9)
    plt.show()


def plot_cumulative_variance(cumulative_explained_variance):
    plt.plot(range(1, len(cumulative_explained_variance) + 1), cumulative_explained_variance, marker='o',
             linestyle='--')
    plt.title('Varianza Explicada Acumulada')
    plt.xlabel('Número de Componentes Principales')
    plt.ylabel('Varianza Explicada Acumulada')
    plt.grid(True)
    plt.xticks(np.arange(1, len(cumulative_explained_variance) + 1, step=1))
    plt.show()


def print_pca_metrics(pca):
    # Crear DataFrame con las métricas calculadas
    pca_metrics = pd.DataFrame({
        'Standard deviation': pca.explained_variance_,
        'Proportion of Variance': pca.explained_variance_ratio_,
        'Cumulative Proportion': pca.explained_variance_ratio_.cumsum()
    }, index=[f'PC{i + 1}' for i in range(pca.n_components_)]).transpose()

    # Imprimir las métricas calculadas
    print(pca_metrics.to_string())


def get_vip_vars(pca, columns, num_components):
    # Obtener los vectores propios (componentes principales)
    eigen_vectors = pca.components_

    # Identificar las variables más importantes (variables con cargas más altas)
    # Valor absoluto de las cargas de los vectores propios
    absolute_loadings = np.abs(eigen_vectors)

    # Suma de las cargas de cada variable a través de los componentes principales
    importance_scores = np.sum(absolute_loadings, axis=0)

    # Indices de las variables más importantes ordenadas de mayor a menor importancia
    important_indices = np.argsort(importance_scores)[::-1]

    # Obtener los nombres de las variables desde las columnas proporcionadas
    vip_vars = []
    count = 0
    for idx in important_indices:
        vip_vars.append(columns[idx])
        count += 1
        if count == num_components:
            break

    return vip_vars


def process_pca(pca_df):
    # Inicializar PCA
    num_components = pca_df.shape[1]
    ipca = 0
    while True:
        print(f"PCA{ipca}:")
        pca = PCA(n_components=num_components)
        # Ajustar PCA
        pca.fit_transform(pca_df)
        plot_loadings(pca.components_, num_components, pca_df.columns)
        # Obtener varianza explicada
        explained_variance = pca.explained_variance_ratio_
        # Obtener varianza explicada acumulativa
        cumulative_explained_variance = np.cumsum(explained_variance)
        cumulative_explained_variance = pca.explained_variance_ratio_.cumsum()
        # Obtener variables importantes
        vip_vars = get_vip_vars(pca, pca_df.columns, num_components)
        # Imprimir variables importantes
        print(f"Las {num_components} variables más importantes:", vip_vars)
        # Filtrar DataFrame con variables importantes
        pca_df = pca_df[vip_vars]
        # Plot de varianza explicada acumulativa
        plot_cumulative_variance(cumulative_explained_variance)
        # Plot de componentes PCA
        plot_pca_components(pca, pca_df.columns, -0.5, 0.5, -0.5, 0.75)
        # Ajustar PCA nuevamente con variables seleccionadas
        pca.fit_transform(pca_df)
        # Imprimir métricas PCA
        print_pca_metrics(pca)
        if num_components == np.argmax(pca.explained_variance_ratio_.cumsum() >= 0.95) + 1:
            return pca_df, pca
        else:
            num_components = np.argmax(pca.explained_variance_ratio_.cumsum() >= 0.95) + 1
            ipca += 1
