import pickle
import warnings

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
from sklearn.metrics import accuracy_score, classification_report, confusion_matrix
from sklearn.metrics import mean_squared_error, mean_absolute_error, r2_score
from sklearn.model_selection import cross_val_score
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import LabelEncoder

from TOOLS.ml_func import obtener_valoracion, windcaddyalg
from constants import MODELS_DIR, MODELS_CAT, FOLDS, RANDOM_STATE


def plot_columns_df(df):
    for column in df.columns:
        plot_column_distribution(df, column)


def plot_column_distribution(df, column_name):
    plt.figure(figsize=(10, 6))
    sns.histplot(df[column_name], kde=True).lines[0].set_color('crimson')
    plt.title(f'Distribución de {column_name}')
    plt.xlabel(column_name)
    plt.ylabel('Frecuencia')
    plt.grid(True)
    plt.show()


def apply_windcaddyalg(version, row):
    pred_valoracion, pred_sail_size, pred_wind_board_size = windcaddyalg(
        version,
        row['peso_rider'], row['velocidad_viento'],
        row['edad_rider'], row['racha'], row['periodo_medio_oleaje']
    )
    row['pred_valoracion'] = pred_valoracion
    row['pred_sail_size'] = pred_sail_size
    row['pred_wind_board_size'] = pred_wind_board_size
    return row


def evaluate_predictions(pred, truth, var_type):
    if var_type == 'cat':
        # Evaluación categórica
        categorical_accuracy = accuracy_score(truth, pred)
        categorical_report = classification_report(truth, pred)
        print("Categorical Evaluation")
        print(f"Accuracy: {categorical_accuracy}")
        print(categorical_report)
    elif var_type == 'cont':
        # Evaluación continua
        mse = mean_squared_error(truth, pred)
        mae = mean_absolute_error(truth, pred)
        r2 = r2_score(truth, pred)
        print("Continuous Evaluation")
        print(f"MSE: {mse}")
        print(f"MAE: {mae}")
        print(f"R²: {r2}")


def evaluate_models_cat(x_train, x_test, y_train, y_test):
    warnings.filterwarnings("ignore")
    models = MODELS_CAT

    for model_name, model in models.items():
        # # Codificar las clases de destino para XGBClassifier
        if model_name == 'XGB Classifier':
            label_encoder = LabelEncoder()
            y_train_encoded = label_encoder.fit_transform(y_train)
            y_test_encoded = label_encoder.transform(y_test)
        else:
            y_train_encoded = y_train
            y_test_encoded = y_test

        model.fit(x_train, y_train_encoded)
        y_pred = model.predict(x_test)
        evaluate_predictions(y_pred, y_test_encoded, 'cat')
        plot_confusion_matrix(model_name, y_test_encoded, y_pred,
                              classes=label_encoder.classes_
                              if model_name == 'XGB Classifier'
                              else model.classes_)
        # Cross Validation scores
        cv_scores = cross_val_score(model, x_test, y_test_encoded, cv=FOLDS)
        print(f"Cross-Validation Scores: {cv_scores}")
        print(f"Mean CV Score: {cv_scores.mean()}")
        with open(MODELS_DIR + "v2_v3_" + model_name + "_" + "valoracion" + ".pkl", "wb") as file:
            pickle.dump(model, file)


def plot_confusion_matrix(model_name, truth, pred, classes):
    print("Confusion Matrix:", model_name)
    cm = confusion_matrix(truth, pred)
    print(cm)
    labels = np.array([obtener_valoracion(int(label)) for label in classes])
    df_cm = pd.DataFrame(cm, index=labels, columns=labels)
    plt.figure(figsize=(10, 7))
    sns.heatmap(df_cm, annot=True, fmt="d", cmap="Blues")
    plt.xlabel("Predicted")
    plt.ylabel("Truth")
    plt.title("Confusion matrix: " + model_name)
    plt.show()


def split_dataset(df, target):
    X = df.drop(target, axis=1)
    y = df[target]
    return train_test_split(X, y, test_size=0.2, random_state=RANDOM_STATE, stratify=y)
